Packet

Struct Packet 

pub struct Packet { /* private fields */ }
An immutable, contiguous sequence of bytes, specialized for networking applications.

Implementations

Source

impl Packet

pub const EMPTY: Packet

The empty packet.

pub fn is_empty(&self) -> bool

Returns true if this Packet has a length of 0.

Examples
let pkt = Packet::from(vec![]);
assert!(pkt.is_empty());
pub fn iter(&self) -> Iter<'_, u8>

Returns an iterator over the bytes in this Packet. The iterator yields all bytes in order from start to end.

Examples
let pkt = Packet::from(vec![0xAA, 0xBB, 0xCC]);
let mut iter = pkt.iter();
assert_eq!(iter.next(), Some(&0xAA));
assert_eq!(iter.next(), Some(&0xBB));
assert_eq!(iter.next(), Some(&0xCC));
assert_eq!(iter.next(), None);
pub fn len(&self) -> usize

Returns the number of bytes contained in this Packet.

Examples
let pkt = Packet::from(vec![1, 2, 3]);
assert_eq!(pkt.len(), 3);