PacketMut

Struct PacketMut 

pub struct PacketMut { /* private fields */ }
A mutable, contiguous, growable sequence of bytes, specialized for networking applications.

Implementations

Source

impl PacketMut

pub fn new(size: usize) -> Self

Constructs a new PacketMut and allocates an underlying buffer of the given size on the heap. The newly-allocated underlying buffer is filled with zero bytes (0u8).

Examples
let mut pkt = PacketMut::new(5);
assert_eq!(pkt.len(), 5);
assert_eq!(pkt.as_ref(), &[0, 0, 0, 0, 0]);
pkt[4] = 42;
assert_eq!(pkt.as_ref(), &[0, 0, 0, 0, 42]);
pub fn with_capacity(size: usize) -> Self

Constructs a new PacketMut with at least the specified capacity. The packet will be able to hold at least size bytes without reallocating.

Note that the packet will have at least the given capacity, but will have a length of zero until bytes are added to it.

Examples
let mut pkt = PacketMut::with_capacity(5);
assert_eq!(pkt.len(), 0);
assert_eq!(pkt.capacity(), 5);
pub fn capacity(&self) -> usize

Returns the number of contiguous bytes the underlying buffer can hold without reallocating.

Examples
let mut pkt = PacketMut::with_capacity(3);
assert_eq!(pkt.len(), 0);
assert_eq!(pkt.capacity(), 3);
pub fn extend_from_slice(&mut self, slice: &[u8])

Appends the given bytes to the end of this PacketMut. The underlying buffer is resized if it does not have enough capacity.

Examples

Extending within the underlying buffer’s capacity increases the length, but not the capacity:

let mut pkt = PacketMut::with_capacity(3);
pkt.extend_from_slice(&[1, 2, 3]);
assert_eq!(pkt.len(), 3);
assert_eq!(pkt.capacity(), 3);

Extending beyond the backing buffer’s capacity triggers a reallocation, changing both the length and the capacity:

let mut pkt = PacketMut::with_capacity(3);
pkt.extend_from_slice(&[1, 2, 3, 4]);
assert_eq!(pkt.len(), 4);
assert_eq!(pkt.capacity(), 8);
pub fn grow_front(&mut self, len: usize)

Add the given number of zero bytes to the front of this [PacketMut]. The underlying buffer is resized if it does not have enough capacity.

pub fn extend_front_from_slice(&mut self, slice: &[u8])

Prepends the given bytes to this PacketMut. The underlying buffer is resized if it does not have enough capacity.

pub fn get<I>(&self, index: I) -> Option<&<I as SliceIndex<[u8]>>::Output>
where I: SliceIndex<[u8]>,

Returns a reference to an element or subslice depending on the type of index.

If given a position, returns a reference to the element at that position or None if out of bounds. If given a range, returns the subslice corresponding to that range, or None if out of bounds.

pub fn get_mut<I>( &mut self, index: I, ) -> Option<&mut <I as SliceIndex<[u8]>>::Output>
where I: SliceIndex<[u8]>,

Returns a mutable reference to an element or subslice depending on the type of index.

If given a position, returns a reference to the element at that position or None if out of bounds. If given a range, returns the subslice corresponding to that range, or None if out of bounds.

pub fn freeze(self) -> Packet

Converts self into an immutable Packet. This is a zero-cost type conversion simply to indicate the returned packet won’t be mutated anymore, allowing the packet to be cheaply cloned and moved between execution contexts (threads/async tasks).

Examples
let mut pkt_mut = PacketMut::with_capacity(4);
pkt_mut.extend_from_slice(b"hello world");
let pkt1 = pkt_mut.freeze();
let pkt2 = pkt1.clone();
assert_eq!(pkt1, pkt2);
let th = std::thread::spawn(move || {
   assert_eq!(&pkt1[..], b"hello world");
});
assert_eq!(&pkt2[..], b"hello world");
th.join().unwrap();
pub fn is_empty(&self) -> bool

Returns true if this PacketMut has a length of 0.

Examples
let pkt = PacketMut::from(&[]);
assert!(pkt.is_empty());
pub fn iter(&self) -> Iter<'_, u8>

Returns an iterator over the bytes in this PacketMut. The iterator yields all bytes in order from start to end.

Examples
let pkt = PacketMut::from(&[0xAA, 0xBB, 0xCC]);
let mut iter = pkt.iter();
assert_eq!(iter.next(), Some(&0xAA));
assert_eq!(iter.next(), Some(&0xBB));
assert_eq!(iter.next(), Some(&0xCC));
assert_eq!(iter.next(), None);
pub fn iter_mut(&mut self) -> IterMut<'_, u8>

Returns an iterator over the bytes in this PacketMut that allows modifying each value. The iterator yields all bytes in order from start to end.

Examples
let mut pkt = PacketMut::from(&[0xAA, 0xBB, 0xCC]);
for byte in pkt.iter_mut() {
    *byte += 1;
}
assert_eq!(pkt.as_ref(), &[0xAB, 0xBC, 0xCD]);
pub fn len(&self) -> usize

Returns the number of bytes contained in this PacketMut.

Examples
let pkt = PacketMut::from(&[1, 2, 3]);
assert_eq!(pkt.len(), 3);
pub fn truncate(&mut self, count: usize)

Removes the last count bytes from the end of this PacketMut, leaving self.len() - count bytes in the packet. Existing capacity is preserved and the backing buffer is not changed.

Examples
let mut pkt = PacketMut::from(&[1, 2, 3, 4, 5]);
pkt.truncate(3);
assert_eq!(pkt, PacketMut::from(&[1, 2, 3]));
pub fn truncate_front(&mut self, count: usize)

Removes the first count bytes from the front of this PacketMut, leaving self.len() - count bytes in the packet. Existing capacity is preserved and the backing buffer is not changed.

Panics

Panics if at > self.len().

Examples
let mut pkt = PacketMut::from(&[1, 2, 3, 4, 5]);
pkt.truncate_front(2);
assert_eq!(pkt, PacketMut::from(&[3, 4, 5]));
pub fn split_off(&mut self, at: usize) -> PacketMut

Splits the PacketMut into two at the given index.

After the call, self will contain the bytes [0, at), and the returned PacketMut will contain the bytes [at, capacity). Existing capacity is preserved, the backing buffer is not changed, and both self and the returned PacketMut share the same backing buffer.

Panics

Panics if at > len.

Complexity

O(1). Indices are adjusted and reference counts are updated, but none of the backing buffer is traversed or cloned.

Examples
let mut pkt1 = PacketMut::from(&[1, 2, 3, 4, 5]);
let pkt2 = pkt1.split_off(2);
assert_eq!(pkt1, PacketMut::from(&[1, 2]));
assert_eq!(pkt2, PacketMut::from(&[3, 4, 5]));
pub fn split_to(&mut self, at: usize) -> PacketMut

Splits the PacketMut into two at the given index.

After the call, self will contain the bytes [at, len), and the returned PacketMut will contain the bytes [0, at). Existing capacity is preserved, the backing buffer is not changed, and both self and the returned PacketMut share the same backing buffer.

Panics

Panics if at > len.

Complexity

O(1). Indices are adjusted and reference counts are updated, but none of the backing buffer is traversed or cloned.

Examples
let mut pkt1 = PacketMut::from(&[1, 2, 3, 4, 5]);
let pkt2 = pkt1.split_to(2);
assert_eq!(pkt1, PacketMut::from(&[3, 4, 5]));
assert_eq!(pkt2, PacketMut::from(&[1, 2]));
pub fn get_src_addr(&self) -> Option<IpAddr>

Returns the source IP address of the packet.

Returns None if the packet structure doesn’t match an IPv4 or IPv6 datagram.

pub fn get_dst_addr(&self) -> Option<IpAddr>

Returns the destination IP address of the packet.

Returns None if the packet structure doesn’t match an IPv4 or IPv6 datagram.